#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>

/****************************************************************************
    FUNCTION: loadWaveFile
    PURPOSE:  load a wave file and prepare it for playing
****************************************************************************/
/* begin playing out the message */
int loadWaveFile (DWORD dwDevID, char *szWaveFile, LPWAVEHDR FAR * lplpWaveHdr, 
						PCMWAVEFORMAT FAR * FAR *lplpWaveFormat, 
						int _huge * FAR *lphpWaveData, HWAVEOUT FAR *lphWave, 
						HWND hWnd)
{
HMMIO                   hmmio;
MMCKINFO                mmckinfoParent;
MMCKINFO                mmckinfoSubchunk;
DWORD                   dwFmtSize;
DWORD                   dwDataSize;
	
	/* Open wave file */
	hmmio = mmioOpen(szWaveFile, NULL, MMIO_READ | MMIO_ALLOCBUF);
	if(!hmmio)
	{
		MessageBox (NULL, "Could not open wave file", "", MB_OK);
		return -1;
	}

	/* Locate a 'RIFF' chunk with a 'WAVE' form type */
	mmckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E');
	if (mmioDescend(hmmio, (LPMMCKINFO) &mmckinfoParent, NULL, MMIO_FINDRIFF))
	{
		mmioClose(hmmio, 0);
		MessageBox (NULL, "Not a WAV file", "", MB_OK);
		return -1;
	}
	
	/* Find the format chunk */
	mmckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	if (mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK))
	{
		mmioClose(hmmio, 0);
		MessageBox (NULL, "No format chunk", "", MB_OK);
		return -1;
	}

	/* Get the size of the format chunk, allocate memory for it */
	dwFmtSize = mmckinfoSubchunk.cksize;
	*lplpWaveFormat = (PCMWAVEFORMAT FAR *) GlobalAllocPtr(GMEM_MOVEABLE, dwFmtSize);
	if (!*lplpWaveFormat)
	{
		mmioClose(hmmio, 0);
		MessageBox (NULL, "No memory", "", MB_OK);
		return -1;
	}

	/* Read the format chunk */
	if (mmioRead(hmmio, (BYTE _huge *) *lplpWaveFormat, dwFmtSize) != (LONG) dwFmtSize)
	{
		GlobalFreePtr( *lplpWaveFormat );
		MessageBox (NULL, "Error reading format chunk", "", MB_OK);
		mmioClose(hmmio, 0);
		return -1;
	}

	/* make sure the file is 16 bit audio */
	if ((*lplpWaveFormat)->wBitsPerSample != 16) {
		GlobalFreePtr( *lplpWaveFormat );
		MessageBox (NULL, "Not a 16 bit audio file", "", MB_OK);
		mmioClose(hmmio, 0);
		return -1;
	}

	
	/* Ascend out of the format subchunk */
	mmioAscend(hmmio, &mmckinfoSubchunk, 0);
	
	/* Find the data subchunk */
	mmckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	if (mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK))
	{
		MessageBox (NULL, "No data chunk", "", MB_OK);
		GlobalFreePtr( *lplpWaveFormat );
		mmioClose(hmmio, 0);
		return -1;
	}

	/* Get the size of the data subchunk */
	dwDataSize = mmckinfoSubchunk.cksize;
	if (dwDataSize == 0L)
	{
		MessageBox (NULL, "No sound data", "", MB_OK);
		GlobalFreePtr( *lplpWaveFormat );
		mmioClose(hmmio, 0);
		return -1;
	}
		
	/* Allocate and lock memory for the waveform data. */
	*lphpWaveData = (int _huge *) GlobalAllocPtr(GMEM_MOVEABLE | GMEM_SHARE, dwDataSize );
	if (!*lphpWaveData)
	{
		MessageBox (NULL, "Not enough memory", "", MB_OK);
		GlobalFreePtr( *lplpWaveFormat );
		mmioClose(hmmio, 0);
		return -1;
	}

	/* Read the waveform data subchunk */
	if(mmioRead(hmmio, (HPSTR) *lphpWaveData, dwDataSize) != (LONG) dwDataSize)
	{
		MessageBox (NULL, "Error reading data", "", MB_OK);
		GlobalFreePtr(*lplpWaveFormat);
		GlobalFreePtr(*lphpWaveData);
		mmioClose(hmmio, 0);
		return -1;
	}

	/* We're done with the file, close it */
	mmioClose(hmmio, 0);

	/* Allocate a waveform data header */
	*lplpWaveHdr = (WAVEHDR FAR*) GlobalAllocPtr(GMEM_MOVEABLE | GMEM_SHARE, (DWORD)sizeof(WAVEHDR));
	if (!*lplpWaveHdr)
	{
		MessageBox (NULL, "No memory", "", MB_OK);
		GlobalFreePtr(*lphpWaveData);
		GlobalFreePtr(*lplpWaveFormat);
		return -1;
	}

	/* Set up WAVEHDR structure and prepare it to be written to wave device */
	(*lplpWaveHdr)->lpData = (HPSTR)*lphpWaveData;
	(*lplpWaveHdr)->dwBufferLength = dwDataSize;
	(*lplpWaveHdr)->dwFlags = 0L;
	(*lplpWaveHdr)->dwLoops = 0L;
	(*lplpWaveHdr)->dwUser  = 0L;

	/* make sure wave device can play our format */
	if (waveOutOpen((LPHWAVEOUT)NULL,
					(WORD)dwDevID,
					(LPWAVEFORMAT)*lplpWaveFormat, 
					0L, 0L, WAVE_FORMAT_QUERY)) {
		GlobalFreePtr( *lplpWaveFormat );
		GlobalFreePtr(*lphpWaveData);
		MessageBox (NULL, "Unsupported wave format.", "", MB_OK);
		return -1;
	}

	/* open the wave device corresponding to the line */
	if (waveOutOpen (lphWave, 
						(WORD)dwDevID, 
						(LPWAVEFORMAT)*lplpWaveFormat, 
						(DWORD)hWnd, 
						0L, 
						CALLBACK_WINDOW))
	{
		GlobalFreePtr( *lplpWaveFormat );
		GlobalFreePtr(*lphpWaveData);
		MessageBox (NULL, "Error opening wave device.", "", MB_OK);
		return -1;
	}

	/* prepare the wave header for playing */
	if (waveOutPrepareHeader (*lphWave, *lplpWaveHdr, sizeof(WAVEHDR))) {
		GlobalFreePtr( *lplpWaveFormat );
		GlobalFreePtr(*lphpWaveData);
		waveOutClose(*lphWave);
	    MessageBox (NULL, "Error preparing message header.", "", MB_OK);
		return -1;
	}

	return 0;

} /* end function (loadWaveFile) */
/****************************************************************************
    FUNCTION: cleanUpWave
    PURPOSE:  clean up after loadWaveFile
****************************************************************************/
void cleanUpWave (HWAVEOUT hWave, LPWAVEHDR lpWaveHdr, 
						int _huge * hpWaveData)
{
			/* Unprepare the header */
			waveOutUnprepareHeader(hWave, lpWaveHdr, sizeof(WAVEHDR));
			/* close wave device */
			waveOutClose(hWave);
			/* free the wave data */
			GlobalFreePtr (hpWaveData);
			/* free the header */
			GlobalFreePtr (lpWaveHdr);

} /* end function (cleanUpWave) */
